@@ -34,20 +34,17 @@ from keystoneauth1 import access
 from keystoneauth1 import adapter
 from keystoneauth1.identity import base
 from keystoneauth1 import discover
+from oslo_utils import encodeutils
+from oslo_utils import importutils
+from oslo_utils import strutils
+osprofiler_web = importutils.try_import("osprofiler.web")  # noqa
 import requests
+import six.moves.urllib.parse as urlparse
 
 from cinderclient import api_versions
 from cinderclient import exceptions
 import cinderclient.extension
 from cinderclient._i18n import _
-from cinderclient._i18n import _LW
-from oslo_utils import encodeutils
-from oslo_utils import importutils
-from oslo_utils import strutils
-
-osprofiler_web = importutils.try_import("osprofiler.web")
-
-import six.moves.urllib.parse as urlparse
 
 try:
     from eventlet import sleep
@@ -66,6 +63,7 @@ V1_SERVICE_TYPE = 'volume'
 SERVICE_TYPES = {'1': V1_SERVICE_TYPE,
                  '2': V2_SERVICE_TYPE,
                  '3': V3_SERVICE_TYPE}
+REQ_ID_HEADER = 'X-OpenStack-Request-ID'
 
 # tell keystoneclient that we can ignore the /v1|v2/{project_id} component of
 # the service catalog when doing discovery lookups
@@ -92,8 +90,8 @@ def get_server_version(url):
                 return (api_versions.APIVersion(version['min_version']),
                         api_versions.APIVersion(version['version']))
     except exceptions.ClientException as e:
-        logger.warning(_LW("Error in server version query:%s\n"
-                 "Returning APIVersion 2.0"), six.text_type(e.message))
+        logger.warning("Error in server version query:%s\n"
+                       "Returning APIVersion 2.0", six.text_type(e.message))
         return api_versions.APIVersion("2.0"), api_versions.APIVersion("2.0")
 
 
@@ -128,10 +126,12 @@ def _log_request_id(logger, resp, service_name):
 
 
 class SessionClient(adapter.LegacyJsonAdapter):
+    global_request_id = None
 
     def __init__(self, *args, **kwargs):
         self.api_version = kwargs.pop('api_version', None)
         self.api_version = self.api_version or api_versions.APIVersion()
+        self.global_request_id = kwargs.pop('global_request_id', None)
         self.retries = kwargs.pop('retries', 0)
         self._logger = logging.getLogger(__name__)
         super(SessionClient, self).__init__(*args, **kwargs)
@@ -140,6 +140,10 @@ class SessionClient(adapter.LegacyJsonAdapter):
         kwargs.setdefault('headers', kwargs.get('headers', {}))
         api_versions.update_headers(kwargs["headers"], self.api_version)
         kwargs.setdefault('authenticated', False)
+
+        if self.global_request_id:
+            kwargs['headers'].setdefault(REQ_ID_HEADER, self.global_request_id)
+
         # Note(tpatil): The standard call raises errors from
         # keystoneauth, here we need to raise the cinderclient errors.
         raise_exc = kwargs.pop('raise_exc', True)
@@ -234,12 +238,13 @@ class HTTPClient(object):
                  http_log_debug=False, cacert=None,
                  auth_system='keystone', auth_plugin=None, api_version=None,
                  logger=None, user_domain_name='Default',
-                 project_domain_name='Default'):
+                 project_domain_name='Default', global_request_id=None):
         self.user = user
         self.password = password
         self.projectid = projectid
         self.tenant_id = tenant_id
         self.api_version = api_version or api_versions.APIVersion()
+        self.global_request_id = global_request_id
 
         if auth_system and auth_system != 'keystone' and not auth_plugin:
             raise exceptions.AuthSystemNotFound(auth_system)
@@ -338,6 +343,9 @@ class HTTPClient(object):
             kwargs['data'] = json.dumps(kwargs.pop('body'))
         api_versions.update_headers(kwargs["headers"], self.api_version)
 
+        if self.global_request_id:
+            kwargs['headers'].setdefault(REQ_ID_HEADER, self.global_request_id)
+
         if self.timeout:
             kwargs.setdefault('timeout', self.timeout)
         self.http_log_req((url, method,), kwargs)
@@ -434,7 +442,7 @@ class HTTPClient(object):
             version = get_volume_api_from_url(self.management_url)
         except exceptions.UnsupportedVersion as e:
             if self.management_url == self.bypass_url:
-                msg = (_("Invalid url was specified in --bypass-url or "
+                msg = (_("Invalid url was specified in --os-endpoint or "
                          "environment variable CINDERCLIENT_BYPASS_URL.\n"
                          "%s") % six.text_type(e.message))
             else:
@@ -532,8 +540,6 @@ class HTTPClient(object):
             while auth_url:
                 if not self.auth_system or self.auth_system == 'keystone':
                     auth_url = self._v2_or_v3_auth(auth_url)
-                else:
-                    auth_url = self._plugin_auth(auth_url)
 
             # Are we acting on behalf of another user via an
             # existing token? If so, our actual endpoints may
@@ -587,9 +593,6 @@ class HTTPClient(object):
         else:
             raise exceptions.from_response(resp, body)
 
-    def _plugin_auth(self, auth_url):
-        return self.auth_plugin.authenticate(self, auth_url)
-
     def _v2_or_v3_auth(self, url):
         """Authenticate against a v2.0 auth service."""
         if self.version == "v3":
@@ -651,8 +654,7 @@ def _construct_http_client(username=None, password=None, project_id=None,
                            auth=None, api_version=None,
                            **kwargs):
 
-    # Don't use sessions if third party plugin or bypass_url being used
-    if session and not auth_plugin and not bypass_url:
+    if session:
         kwargs.setdefault('user_agent', 'python-cinderclient')
         kwargs.setdefault('interface', endpoint_type)
         return SessionClient(session=session,
