@@ -32,11 +32,149 @@ from cinderclient import utils
 
 from cinderclient.v2.shell import *  # flake8: noqa
 
+FILTER_DEPRECATED = ("This option is deprecated and will be removed in "
+                     "newer release. Please use '--filters' option which "
+                     "is introduced since 3.33 instead.")
+
+
+@api_versions.wraps('3.33')
+@utils.arg('--resource',
+           metavar='<resource>',
+           default=None,
+           help='Show enabled filters for specified resource. Default=None.')
+def do_list_filters(cs, args):
+        filters = cs.resource_filters.list(resource=args.resource)
+        shell_utils.print_resource_filter_list(filters)
+
+
+@utils.arg('--all-tenants',
+           metavar='<all_tenants>',
+           nargs='?',
+           type=int,
+           const=1,
+           default=0,
+           help='Shows details for all tenants. Admin only.')
+@utils.arg('--all_tenants',
+           nargs='?',
+           type=int,
+           const=1,
+           help=argparse.SUPPRESS)
+@utils.arg('--name',
+           metavar='<name>',
+           default=None,
+           help="Filters results by a name. Default=None. "
+                "%s" % FILTER_DEPRECATED)
+@utils.arg('--status',
+           metavar='<status>',
+           default=None,
+           help="Filters results by a status. Default=None. "
+                "%s" % FILTER_DEPRECATED)
+@utils.arg('--volume-id',
+           metavar='<volume-id>',
+           default=None,
+           help="Filters results by a volume ID. Default=None. "
+                "%s" % FILTER_DEPRECATED)
+@utils.arg('--volume_id',
+           help=argparse.SUPPRESS)
+@utils.arg('--marker',
+           metavar='<marker>',
+           default=None,
+           help='Begin returning backups that appear later in the backup '
+                'list than that represented by this id. '
+                'Default=None.')
+@utils.arg('--limit',
+           metavar='<limit>',
+           default=None,
+           help='Maximum number of backups to return. Default=None.')
+@utils.arg('--sort',
+           metavar='<key>[:<direction>]',
+           default=None,
+           help=(('Comma-separated list of sort keys and directions in the '
+                  'form of <key>[:<asc|desc>]. '
+                  'Valid keys: %s. '
+                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
+def do_backup_list(cs, args):
+    """Lists all backups."""
+    # pylint: disable=function-redefined
+
+    search_opts = {
+        'all_tenants': args.all_tenants,
+        'name': args.name,
+        'status': args.status,
+        'volume_id': args.volume_id,
+    }
+
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
+
+    backups = cs.backups.list(search_opts=search_opts,
+                              marker=args.marker,
+                              limit=args.limit,
+                              sort=args.sort)
+    shell_utils.translate_volume_snapshot_keys(backups)
+    columns = ['ID', 'Volume ID', 'Status', 'Name', 'Size', 'Object Count',
+               'Container']
+    if args.sort:
+        sortby_index = None
+    else:
+        sortby_index = 0
+    utils.print_list(backups, columns, sortby_index=sortby_index)
+
+
+@utils.arg('--detail',
+           action='store_true',
+           help='Show detailed information about pools.')
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
+def do_get_pools(cs, args):
+    """Show pool information for backends. Admin only."""
+    # pylint: disable=function-redefined
+    search_opts = {}
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
+    if cs.api_version >= api_versions.APIVersion("3.33"):
+        pools = cs.volumes.get_pools(args.detail, search_opts)
+    else:
+        pools = cs.volumes.get_pools(args.detail)
+    infos = dict()
+    infos.update(pools._info)
+
+    for info in infos['pools']:
+        backend = dict()
+        backend['name'] = info['name']
+        if args.detail:
+            backend.update(info['capabilities'])
+        utils.print_dict(backend)
+
+
+RESET_STATE_RESOURCES = {'volume': utils.find_volume,
+                         'backup': shell_utils.find_backup,
+                         'snapshot': shell_utils.find_volume_snapshot,
+                         'group': shell_utils.find_group,
+                         'group-snapshot': shell_utils.find_group_snapshot}
+
 
 @utils.arg('--group_id',
            metavar='<group_id>',
            default=None,
-           help='Filters results by a group_id. Default=None.',
+           help="Filters results by a group_id. Default=None."
+                "%s" % FILTER_DEPRECATED,
            start_version='3.10')
 @utils.arg('--all-tenants',
            dest='all_tenants',
@@ -54,39 +192,43 @@ from cinderclient.v2.shell import *  # flake8: noqa
 @utils.arg('--name',
            metavar='<name>',
            default=None,
-           help='Filters results by a name. Default=None.')
+           help="Filters results by a name. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--display-name',
            help=argparse.SUPPRESS)
 @utils.arg('--status',
            metavar='<status>',
            default=None,
-           help='Filters results by a status. Default=None.')
+           help="Filters results by a status. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--bootable',
            metavar='<True|true|False|false>',
            const=True,
            nargs='?',
            choices=['True', 'true', 'False', 'false'],
-           help='Filters results by bootable status. Default=None.')
+           help="Filters results by bootable status. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--migration_status',
            metavar='<migration_status>',
            default=None,
-           help='Filters results by a migration status. Default=None. '
-                'Admin only.')
+           help="Filters results by a migration status. Default=None. "
+                "Admin only. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--metadata',
-           type=str,
            nargs='*',
            metavar='<key=value>',
            default=None,
-           help='Filters results by a metadata key and value pair. '
-                'Default=None.')
+           help="Filters results by a metadata key and value pair. "
+                "Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--image_metadata',
-           type=str,
            nargs='*',
            metavar='<key=value>',
            default=None,
            start_version='3.4',
-           help='Filters results by a image metadata key and value pair. Require '
-                'volume api version >=3.4. Default=None.')
+           help="Filters results by a image metadata key and value pair. "
+                "Require volume api version >=3.4. Default=None."
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--marker',
            metavar='<marker>',
            default=None,
@@ -125,8 +267,17 @@ from cinderclient.v2.shell import *  # flake8: noqa
            nargs='?',
            metavar='<tenant>',
            help='Display information from single tenant (Admin only).')
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
 def do_list(cs, args):
     """Lists all volumes."""
+    # pylint: disable=function-redefined
     # NOTE(thingee): Backwards-compatibility with v1 args
     if args.display_name is not None:
         args.name = args.display_name
@@ -147,6 +298,9 @@ def do_list(cs, args):
         if hasattr(args, 'image_metadata') and args.image_metadata else None,
         'group_id': getattr(args, 'group_id', None),
     }
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
 
     # If unavailable/non-existent fields are specified, these fields will
     # be removed from key_list at the print_list() during key validation.
@@ -194,6 +348,64 @@ def do_list(cs, args):
                      sortby_index=sortby_index)
 
 
+@utils.arg('entity', metavar='<entity>', nargs='+',
+           help='Name or ID of entity to update.')
+@utils.arg('--type', metavar='<type>', default='volume',
+           choices=RESET_STATE_RESOURCES.keys(),
+           help="Type of entity to update. Available resources "
+                "are: 'volume', 'snapshot', 'backup', "
+                "'group' (since 3.20) and "
+                "'group-snapshot' (since 3.19), Default=volume.")
+@utils.arg('--state', metavar='<state>', default=None,
+           help=("The state to assign to the entity. "
+                 "NOTE: This command simply changes the state of the "
+                 "entity in the database with no regard to actual status, "
+                 "exercise caution when using. Default=None, that means the "
+                 "state is unchanged."))
+@utils.arg('--attach-status', metavar='<attach-status>', default=None,
+           help=('This is only used for a volume entity. The attach status '
+                 'to assign to the volume in the database, with no regard '
+                 'to the actual status. Valid values are "attached" and '
+                 '"detached". Default=None, that means the status '
+                 'is unchanged.'))
+@utils.arg('--reset-migration-status',
+           action='store_true',
+           help=('This is only used for a volume entity. Clears the migration '
+                 'status of the volume in the DataBase that indicates the '
+                 'volume is source or destination of volume migration, '
+                 'with no regard to the actual status.'))
+def do_reset_state(cs, args):
+    """Explicitly updates the entity state in the Cinder database.
+
+    Being a database change only, this has no impact on the true state of the
+    entity and may not match the actual state. This can render a entity
+    unusable in the case of changing to the 'available' state.
+    """
+    # pylint: disable=function-redefined
+    failure_count = 0
+    single = (len(args.entity) == 1)
+
+    migration_status = 'none' if args.reset_migration_status else None
+    collector = RESET_STATE_RESOURCES[args.type]
+    argument = (args.state,)
+    if args.type == 'volume':
+        argument += (args.attach_status, migration_status)
+
+    for entity in args.entity:
+        try:
+            collector(cs, entity).reset_state(*argument)
+        except Exception as e:
+            print(e)
+            failure_count += 1
+            msg = "Reset state for entity %s failed: %s" % (entity, e)
+            if not single:
+                print(msg)
+
+    if failure_count == len(args.entity):
+        msg = "Unable to reset the state for the specified entity(s)."
+        raise exceptions.CommandError(msg)
+
+
 @utils.arg('size',
            metavar='<size>',
            nargs='?',
@@ -269,7 +481,6 @@ def do_list(cs, args):
 @utils.arg('--availability_zone',
            help=argparse.SUPPRESS)
 @utils.arg('--metadata',
-           type=str,
            nargs='*',
            metavar='<key=value>',
            default=None,
@@ -469,7 +680,7 @@ def do_group_type_delete(cs, args):
             gtype = shell_utils.find_group_type(cs, group_type)
             cs.group_types.delete(gtype)
             print("Request to delete group type %s has been accepted."
-                  % (group_type))
+                  % group_type)
         except Exception as e:
             failure_count += 1
             print("Delete for group type %s failed: %s" % (group_type, e))
@@ -717,7 +928,8 @@ def do_cluster_disable(cs, args):
            metavar='<marker>',
            default=None,
            help='Begin returning volumes that appear later in the volume '
-                'list than that represented by this volume id. '
+                'list than that represented by this reference. This reference '
+                'should be json like. '
                 'Default=None.')
 @utils.arg('--limit',
            metavar='<limit>',
@@ -733,9 +945,11 @@ def do_cluster_disable(cs, args):
            help=(('Comma-separated list of sort keys and directions in the '
                   'form of <key>[:<asc|desc>]. '
                   'Valid keys: %s. '
-                  'Default=None.') % ', '.join(base.SORT_MANAGEABLE_KEY_VALUES)))
+                  'Default=None.'
+                  ) % ', '.join(base.SORT_MANAGEABLE_KEY_VALUES)))
 def do_manageable_list(cs, args):
     """Lists all manageable volumes."""
+    # pylint: disable=function-redefined
     detailed = strutils.bool_from_string(args.detailed)
     volumes = cs.volumes.list_manageable(host=args.host, detailed=detailed,
                                          marker=args.marker, limit=args.limit,
@@ -755,10 +969,22 @@ def do_manageable_list(cs, args):
            const=1,
            default=utils.env('ALL_TENANTS', default=0),
            help='Shows details for all tenants. Admin only.')
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
 def do_group_list(cs, args):
     """Lists all groups."""
     search_opts = {'all_tenants': args.all_tenants}
 
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
+
     groups = cs.groups.list(search_opts=search_opts)
 
     columns = ['ID', 'Status', 'Name']
@@ -938,11 +1164,21 @@ def do_group_update(cs, args):
 @utils.arg('--status',
            metavar='<status>',
            default=None,
-           help='Filters results by a status. Default=None.')
+           help="Filters results by a status. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--group-id',
            metavar='<group_id>',
            default=None,
-           help='Filters results by a group ID. Default=None.')
+           help="Filters results by a group ID. Default=None. "
+                "%s" % FILTER_DEPRECATED)
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
 def do_group_snapshot_list(cs, args):
     """Lists all group snapshots."""
 
@@ -953,6 +1189,9 @@ def do_group_snapshot_list(cs, args):
         'status': args.status,
         'group_id': args.group_id,
     }
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
 
     group_snapshots = cs.group_snapshots.list(search_opts=search_opts)
 
@@ -1068,7 +1307,8 @@ def do_service_list(cs, args):
            metavar='<marker>',
            default=None,
            help='Begin returning volumes that appear later in the volume '
-                'list than that represented by this volume id. '
+                'list than that represented by this reference. This reference '
+                'should be json like. '
                 'Default=None.')
 @utils.arg('--limit',
            metavar='<limit>',
@@ -1084,9 +1324,11 @@ def do_service_list(cs, args):
            help=(('Comma-separated list of sort keys and directions in the '
                   'form of <key>[:<asc|desc>]. '
                   'Valid keys: %s. '
-                  'Default=None.') % ', '.join(base.SORT_MANAGEABLE_KEY_VALUES)))
+                  'Default=None.'
+                  ) % ', '.join(base.SORT_MANAGEABLE_KEY_VALUES)))
 def do_snapshot_manageable_list(cs, args):
     """Lists all manageable snapshots."""
+    # pylint: disable=function-redefined
     detailed = strutils.bool_from_string(args.detailed)
     snapshots = cs.volume_snapshots.list_manageable(host=args.host,
                                                     detailed=detailed,
@@ -1107,6 +1349,7 @@ def do_api_version(cs, args):
     response = cs.services.server_api_version()
     utils.print_list(response, columns)
 
+
 @api_versions.wraps("3.3")
 @utils.arg('--marker',
            metavar='<marker>',
@@ -1131,23 +1374,36 @@ def do_api_version(cs, args):
 @utils.arg('--resource_uuid',
            metavar='<resource_uuid>',
            default=None,
-           help='Filters results by a resource uuid. Default=None.')
+           help="Filters results by a resource uuid. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--resource_type',
            metavar='<type>',
            default=None,
-           help='Filters results by a resource type. Default=None.')
+           help="Filters results by a resource type. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--event_id',
            metavar='<id>',
            default=None,
-           help='Filters results by event id. Default=None.')
+           help="Filters results by event id. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--request_id',
            metavar='<request_id>',
            default=None,
-           help='Filters results by request id. Default=None.')
+           help="Filters results by request id. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--level',
            metavar='<level>',
            default=None,
-           help='Filters results by the message level. Default=None.')
+           help="Filters results by the message level. Default=None. "
+                "%s" % FILTER_DEPRECATED)
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
 def do_message_list(cs, args):
     """Lists all messages."""
     search_opts = {
@@ -1155,6 +1411,9 @@ def do_message_list(cs, args):
         'event_id': args.event_id,
         'request_id': args.request_id,
     }
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
     if args.resource_type:
         search_opts['resource_type'] = args.resource_type.upper()
     if args.level:
@@ -1208,6 +1467,7 @@ def do_message_delete(cs, args):
         raise exceptions.CommandError("Unable to delete any of the specified "
                                       "messages.")
 
+
 @utils.arg('--all-tenants',
            dest='all_tenants',
            metavar='<0|1>',
@@ -1224,7 +1484,8 @@ def do_message_delete(cs, args):
 @utils.arg('--name',
            metavar='<name>',
            default=None,
-           help='Filters results by a name. Default=None.')
+           help="Filters results by a name. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--display-name',
            help=argparse.SUPPRESS)
 @utils.arg('--display_name',
@@ -1232,11 +1493,13 @@ def do_message_delete(cs, args):
 @utils.arg('--status',
            metavar='<status>',
            default=None,
-           help='Filters results by a status. Default=None.')
+           help="Filters results by a status. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--volume-id',
            metavar='<volume-id>',
            default=None,
-           help='Filters results by a volume ID. Default=None.')
+           help="Filters results by a volume ID. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--volume_id',
            help=argparse.SUPPRESS)
 @utils.arg('--marker',
@@ -1267,10 +1530,20 @@ def do_message_delete(cs, args):
            metavar='<key=value>',
            default=None,
            start_version='3.22',
-           help='Filters results by a metadata key and value pair. Require '
-                'volume api version >=3.22. Default=None.')
+           help="Filters results by a metadata key and value pair. Require "
+                "volume api version >=3.22. Default=None. "
+                "%s" % FILTER_DEPRECATED)
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
 def do_snapshot_list(cs, args):
     """Lists all snapshots."""
+    # pylint: disable=function-redefined
     all_tenants = (1 if args.tenant else
                    int(os.environ.get("ALL_TENANTS", args.all_tenants)))
 
@@ -1293,6 +1566,10 @@ def do_snapshot_list(cs, args):
         'metadata': metadata
     }
 
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
+
     snapshots = cs.volume_snapshots.list(search_opts=search_opts,
                                          marker=args.marker,
                                          limit=args.limit,
@@ -1303,6 +1580,7 @@ def do_snapshot_list(cs, args):
                      ['ID', 'Volume ID', 'Status', 'Name', 'Size'],
                      sortby_index=sortby_index)
 
+
 @api_versions.wraps('3.27')
 @utils.arg('--all-tenants',
            dest='all_tenants',
@@ -1310,16 +1588,18 @@ def do_snapshot_list(cs, args):
            nargs='?',
            type=int,
            const=1,
-           default=0,
+           default=utils.env('ALL_TENANTS', default=0),
            help='Shows details for all tenants. Admin only.')
 @utils.arg('--volume-id',
            metavar='<volume-id>',
            default=None,
-           help='Filters results by a volume ID. Default=None.')
+           help="Filters results by a volume ID. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--status',
            metavar='<status>',
            default=None,
-           help='Filters results by a status. Default=None.')
+           help="Filters results by a status. Default=None. "
+                "%s" % FILTER_DEPRECATED)
 @utils.arg('--marker',
            metavar='<marker>',
            default=None,
@@ -1329,7 +1609,7 @@ def do_snapshot_list(cs, args):
 @utils.arg('--limit',
            metavar='<limit>',
            default=None,
-           help='Maximum number of attachemnts to return. Default=None.')
+           help='Maximum number of attachments to return. Default=None.')
 @utils.arg('--sort',
            metavar='<key>[:<direction>]',
            default=None,
@@ -1343,19 +1623,31 @@ def do_snapshot_list(cs, args):
            nargs='?',
            metavar='<tenant>',
            help='Display information from single tenant (Admin only).')
+@utils.arg('--filters',
+           type=str,
+           nargs='*',
+           start_version='3.33',
+           metavar='<key=value>',
+           default=None,
+           help="Filter key and value pairs. Please use 'cinder list-filters' "
+                "to check enabled filters from server, Default=None.")
 def do_attachment_list(cs, args):
     """Lists all attachments."""
     search_opts = {
-        'all_tenants': args.all_tenants,
+        'all_tenants': 1 if args.tenant else args.all_tenants,
+        'project_id': args.tenant,
         'status': args.status,
         'volume_id': args.volume_id,
     }
+    # Update search option with `filters`
+    if hasattr(args, 'filters') and args.filters is not None:
+        search_opts.update(shell_utils.extract_filters(args.filters))
 
     attachments = cs.attachments.list(search_opts=search_opts,
                                       marker=args.marker,
                                       limit=args.limit,
                                       sort=args.sort)
-    columns = ['ID', 'Volume ID', 'Status', 'Instance']
+    columns = ['ID', 'Volume ID', 'Status', 'Server ID']
     if args.sort:
         sortby_index = None
     else:
@@ -1385,10 +1677,9 @@ def do_attachment_show(cs, args):
 @utils.arg('volume',
            metavar='<volume>',
            help='Name or ID of volume or volumes to attach.')
-@utils.arg('--instance',
-           metavar='<instance>',
-           default=None,
-           help='UUID of Instance attaching to. Default=None.')
+@utils.arg('server_id',
+           metavar='<server_id>',
+           help='ID of server attaching to.')
 @utils.arg('--connect',
            metavar='<connect>',
            default=False,
@@ -1417,7 +1708,7 @@ def do_attachment_show(cs, args):
 @utils.arg('--multipath',
            metavar='<multipath>',
            default=False,
-           help='OS type. Default=False.')
+           help='Use multipath. Default=False.')
 @utils.arg('--mountpoint',
            metavar='<mountpoint>',
            default=None,
@@ -1433,10 +1724,11 @@ def do_attachment_create(cs, args):
                      'platform': args.platform,
                      'host': args.host,
                      'os_type': args.ostype,
-                     'multipath': args.multipath}
+                     'multipath': args.multipath,
+                     'mountpoint': args.mountpoint}
     attachment = cs.attachments.create(args.volume,
                                        connector,
-                                       args.instance)
+                                       args.server_id)
     connector_dict = attachment.pop('connection_info', None)
     utils.print_dict(attachment)
     if connector_dict:
@@ -1470,7 +1762,7 @@ def do_attachment_create(cs, args):
 @utils.arg('--multipath',
            metavar='<multipath>',
            default=False,
-           help='OS type. Default=False.')
+           help='Use multipath. Default=False.')
 @utils.arg('--mountpoint',
            metavar='<mountpoint>',
            default=None,
@@ -1486,7 +1778,8 @@ def do_attachment_update(cs, args):
                  'platform': args.platform,
                  'host': args.host,
                  'os_type': args.ostype,
-                 'multipath': args.multipath}
+                 'multipath': args.multipath,
+                 'mountpoint': args.mountpoint}
     attachment = cs.attachments.update(args.attachment,
                                        connector)
     attachment_dict = attachment.to_dict()
@@ -1505,6 +1798,7 @@ def do_attachment_delete(cs, args):
     for attachment in args.attachment:
         cs.attachments.delete(attachment)
 
+
 @api_versions.wraps('3.0')
 def do_version_list(cs, args):
     """List all API versions."""
@@ -1514,11 +1808,11 @@ def do_version_list(cs, args):
     else:
         columns = ["Id", "Status", "Updated"]
 
-    print(("Client supported API versions:"))
-    print(("Minimum version %(v)s") %
+    print("Client supported API versions:")
+    print("Minimum version %(v)s" %
           {'v': api_versions.MIN_VERSION})
-    print(("Maximum version %(v)s") %
+    print("Maximum version %(v)s" %
           {'v': api_versions.MAX_VERSION})
 
-    print(("\nServer supported API versions:"))
+    print("\nServer supported API versions:")
     utils.print_list(result, columns)
